﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class Game : MonoBehaviour {

	public enum GameState {
		cpuTurn,
		playerTurn
	}

	public class PlanetInfos {
		public GameObject m_gameObject = null;
		public bool m_deployTracker = false;
		public bool m_trackerDeployed = false;
	};


	public Vector2 m_minMaxPlanetCount = new Vector2( 10, 20 );
	public float m_cpuTurnDuration = 5.0f;

	public GameObject m_planetPrefab = null;
	public GameObject m_sun = null;

	private int m_planetCount = 0;
	private List<float> m_radius = new List<float>( );
	private float m_biggestRadius = 1.0f;
	private Dictionary<string, PlanetInfos> m_planets = new Dictionary<string, PlanetInfos>( );

	public bool m_isPaused = false;
	private string m_playerPosition = "Adelpha";
	private string m_targetPosition = "";
	private int m_trackerCount = 20;
	private GameState m_state = GameState.playerTurn;
	private float m_cpuTurnAge = 0.0f;
	private bool m_firstFrame = true;

	private static Game ms_instance = null;

	public static Game get( ) {
		return ms_instance;
	}

	void Awake( ) {
		ms_instance = this;
	}

	void Start( ) {

		DebugConsole.create( );
		DebugConsole.registerCommand( "listBodies", listBodies );
		DebugConsole.registerCommand( "goTo", playerGoTo );
		DebugConsole.registerCommand( "connect", connect );
		DebugConsole.registerCommand( "signalTarget", signalTarget );
		DebugConsole.registerCommand( "listPlanetsNear", listPlanetsNear );
		DebugConsole.registerCommand( "wait", wait );
		DebugConsole.registerPostCommand( endTurn );

		m_planetCount = Random.Range( ( int ) m_minMaxPlanetCount.x, ( int ) m_minMaxPlanetCount.y );

		for ( int i = 0; i < m_planetCount; i++ ) {

			GameObject p = Instantiate( m_planetPrefab ) as GameObject;
			p.name = "p" + i;
			float diameter = Random.Range( 0.5f, 5.0f );
			p.transform.localScale = new Vector3( diameter, diameter, diameter );
			Vector3 direction = new Vector3( Random.Range( -1.0f, 1.0f ), 0.0f, Random.Range( -1.0f, 1.0f ) ).normalized;
			float radius = 0.0f;

			while ( radius < 10.0f ) {

				radius = Random.Range( 10.0f, 6.0f * m_planetCount );

				foreach ( float r in m_radius ) {

					if ( Mathf.Abs( radius - r ) < diameter * 0.5f + 1.0f ) {
						radius = 0.0f;
						break;
					}
				}
			}

			m_radius.Add( radius );

			if ( radius > m_biggestRadius ) {
				m_biggestRadius = radius;
			}

			p.transform.position = direction * radius;
			p.renderer.material.color = new Color( Random.Range( 0.0f, 1.0f ), Random.Range( 0.0f, 1.0f ), Random.Range( 0.0f, 1.0f ) );

			GameObject d = new GameObject( "d" + i );
			d.transform.position = Vector3.zero;
			p.transform.parent = d.transform;

			m_planets.Add( p.name, new PlanetInfos( ) );
			m_planets[ p.name ].m_gameObject = p;
			m_planets[ p.name ].m_deployTracker = true;
		}

		m_targetPosition = "p" + Random.Range( 0, m_planetCount );
	}

	void Update( ) {

		if ( m_firstFrame ) {

			List<string> pNames = listNearbyPlanets( m_targetPosition, 100.0f );
			DebugConsole.log( "Your target was last seen near " + pNames[ Random.Range( 0, pNames.Count ) ] );
			DebugConsole.log( "Find it, and signal him to the authorita." );
			m_firstFrame = false;
		}

		switch( m_state ) {

			case GameState.playerTurn: {
				m_isPaused = true;
				DebugConsole.showConsole( );
			} break;

			case GameState.cpuTurn: {

				m_isPaused = false;
				DebugConsole.hideConsole( );

				m_cpuTurnAge += Time.deltaTime;

				if ( m_cpuTurnAge > m_cpuTurnDuration ) {

					List<string> destinations = listNearbyPlanets( m_targetPosition, 100.0f );
					m_targetPosition = destinations[ Random.Range( 0, destinations.Count ) ];

					foreach ( PlanetInfos p in m_planets.Values ) {

						if ( p.m_trackerDeployed ) {

							if ( Vector3.Distance( p.m_gameObject.transform.position, m_planets[ m_targetPosition ].m_gameObject.transform.position ) < 50.0f ) {
								DebugConsole.log( "Target signature detected near " + p.m_gameObject.name );
								DebugConsole.log( "Possible destinations: " + p.m_gameObject.name );

								destinations = listNearbyPlanets( p.m_gameObject.name, 50.0f );

								foreach ( string s in destinations ) {
									DebugConsole.log( "- " + s );
								}
							}
						}
					}

					m_state = GameState.playerTurn;
				}

			} break;
		}
	}

	public float getBiggestRadius( ) {
		return m_biggestRadius;
	}

	public bool listBodies( string s ) {

		DebugConsole.log( m_sun.name );

		foreach ( string p in m_planets.Keys ) {
			DebugConsole.log( p );
		}

		return false;
	}

	public bool playerGoTo( string name ) {

		if ( m_planets.ContainsKey( name ) ) {
			m_playerPosition = name;
			DebugConsole.execute( "lookAt", name );
			return true;
		} else {
			DebugConsole.log( "Unknown destination: \"" + name + "\"" );
			return false;
		}
	}

	public bool connect( string s ) {

		if ( m_planets.ContainsKey( m_playerPosition ) ) {

			DebugConsole.log( "> connected to " + m_playerPosition );

			if ( m_planets[ m_playerPosition ].m_deployTracker ) {
				DebugConsole.log( "deployTracker [" + m_trackerCount + " left]" );
				DebugConsole.registerCommand( "deployTracker", deployTracker );
			}

			return true;

		} else {

			DebugConsole.log( "Can not connect from here" );
			return false;
		}
		 
	}

	public bool deployTracker( string s ) {

		if ( m_planets.ContainsKey( m_playerPosition ) && m_trackerCount > 0 ) {
			DebugConsole.log( "Tracker deployed." );
			DebugConsole.fixedLog( "Tracker on " + m_playerPosition );
			m_planets[ m_playerPosition ].m_trackerDeployed = true;
			m_trackerCount--;

			return true;
		}

		return false;
	}

	public bool signalTarget( string s ) {

		if ( s!= null && m_planets.ContainsKey( s ) ) {

			if ( s == m_targetPosition ) {
				DebugConsole.log( "The authorita captured the target. [This is the end]" );
				return false;
			} else {
				DebugConsole.log( "The authorita didn't find the target." );
				return true;
			}

		} else {

			DebugConsole.log( "Unknown location." );
			return false;
		}
	}

	public void endTurn( ) {

		m_state = GameState.cpuTurn;
		m_cpuTurnAge = 0.0f;
	}

	public List<string> listNearbyPlanets( string name, float radius ) {

		List<string> l = new List<string>( );

		foreach ( PlanetInfos p in m_planets.Values ) {

			if ( p.m_gameObject.name != name && Vector3.Distance( p.m_gameObject.transform.position, m_planets[ name ].m_gameObject.transform.position ) < radius ) {
				l.Add( p.m_gameObject.name );
			}
		}

		return l;
	}

	public bool listPlanetsNear( string name ) {

		if ( name != null && m_planets.ContainsKey( name ) ) {

			List<string> l = listNearbyPlanets( name, 50.0f );

			foreach ( string s in l ) {
				DebugConsole.log( s );
			}

		} else {

			DebugConsole.log( "Unknown location: " + name );
		}

		return false;
	}

	public bool wait( string s ) {
		return true;
	}
}
