﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class DebugConsole : MonoBehaviour {

	public delegate bool CommandCallback( string parameters = null );
	public delegate void PostCommandCallback( );

	private static bool ms_opened = false;
	private static List<string> ms_log = new List<string>( );
	private static List<string> ms_fixedLog = new List<string>( );
	private static List<string> ms_frameLog = new List<string>( );
	private static bool ms_mustClearFrameLog = false;
	private static bool ms_scrollLog = false;
	private static Dictionary<string, CommandCallback> ms_commands = new Dictionary<string, CommandCallback>( );
	private static PostCommandCallback ms_postCommand = null;

	private Vector2 m_logScrollPosition = Vector2.zero;
	private Vector2 m_fixedLogScrollPosition = Vector2.zero;
	private Vector2 m_frameLogScrollPosition = Vector2.zero;
	private string m_command = "";
	private List<string> m_commandHistory = new List<string>( );
	private int m_commandHistoryIndex = 0;

	public static void log( string text ) {
		create( );
		ms_log.Add( text );
		ms_scrollLog = true;
	}

	public static void fixedLog( string text ) {
		create( );
		ms_fixedLog.Add( text );
	}

	public static void frameLog( string text ) {
		create( );
		clearFrameLog( );
		ms_frameLog.Add( text );
	}

	private static void clearFrameLog( ) {

		if ( ms_mustClearFrameLog ) {
			ms_frameLog.Clear( );
			ms_mustClearFrameLog = false;
		}
	}

	public static void create( ) {

		if ( DebugAnchor.get( ).GetComponent<DebugConsole>( ) == null ) {
			DebugAnchor.get( ).AddComponent<DebugConsole>( );
		}
	}

	public static void registerCommand( string name, CommandCallback callback ) {

		create( );

		if ( ms_commands.ContainsKey( name ) ) {
			ms_commands[ name ] += callback;
		} else {
			ms_commands.Add( name, callback );
		}
	}

	public static void unregisterCommand( string name, CommandCallback callback ) {

		if ( ms_commands.ContainsKey( name ) ) {
			ms_commands[ name ] -= callback;
		}

	}

	public static void registerPostCommand( PostCommandCallback callback ) {
		ms_postCommand += callback;
	}

	void Start( ) {
		registerCommand( "help", listCommands );
		registerCommand( "?", listCommands );
	}

	void Update( ) {

		clearFrameLog( );

		if ( Input.GetKeyDown( KeyCode.Escape ) ) {
			ms_opened = !ms_opened;
		}

		char[] newLine = new char[] { '\n', '\r' };

		if ( m_command.Length > 1 && m_command.IndexOfAny( newLine ) != -1 ) {

			while ( m_command.IndexOfAny( newLine ) != -1 ) {
				m_command = m_command.Remove( m_command.IndexOfAny( newLine ), 1 );
			}

			string[] commandAndParameters = m_command.Split( new char[] { ' ' }, 2 );
			string command = commandAndParameters[ 0 ];

			if ( ms_commands.ContainsKey( command ) ) {

				log( "> " + m_command );

				bool result = false;

				if ( commandAndParameters.Length == 2 ) {
					result = ms_commands[ command ]( commandAndParameters[ 1 ] );
				} else {
					result = ms_commands[ command ]( );
				}

				if ( result ) {
					ms_postCommand( );
				}

			} else {

				log( "Unknown command: " + m_command );
			}

			m_commandHistory.Add( m_command );
			m_commandHistoryIndex = m_commandHistory.Count - 1;
			m_command = "";
		}
	}

	void LateUpdate( ) {
		ms_mustClearFrameLog = true;
	}

	void OnGUI( ) {

		if ( ms_opened ) {

			float lineHeight = 21;

			Rect logRect = new Rect( 0, 0, Screen.width * 0.5f, Screen.height * 0.5f - lineHeight);
			Rect logScrollRect = new Rect( 0, 0, logRect.width - 16, ms_log.Count * lineHeight );

			if ( ms_scrollLog ) {
				m_logScrollPosition.y = ms_log.Count * lineHeight;
				ms_scrollLog = false;
			}

			m_logScrollPosition = GUI.BeginScrollView( logRect, m_logScrollPosition, logScrollRect );

				for ( int i = 0; i < ms_log.Count; i++ ) {
					GUI.Label( new Rect( 0, i * lineHeight, logScrollRect.width, lineHeight ), ms_log[ i ] );
				}
				
			GUI.EndScrollView( );

			Rect fixedLogRect = new Rect( logRect.width + 5, 0, Screen.width * 0.25f - 5, Screen.height * 0.5f - lineHeight );
			Rect fixedLogScrollRect = new Rect( 0, 0, fixedLogRect.width - 16, ms_fixedLog.Count * lineHeight );

			m_fixedLogScrollPosition = GUI.BeginScrollView( fixedLogRect, m_fixedLogScrollPosition, fixedLogScrollRect );

				for ( int i = 0; i < ms_fixedLog.Count; i++ ) {
					GUI.Label( new Rect( 0, i * lineHeight, fixedLogScrollRect.width, lineHeight ), ms_fixedLog[ i ] );
				}

			GUI.EndScrollView( );

			Rect frameLogRect = new Rect( logRect.width + fixedLogRect.width + 10, 0, Screen.width * 0.25f - 5, Screen.height * 0.5f - lineHeight );
			Rect frameLogScrollRect = new Rect( 0, 0, frameLogRect.width - 16, ms_frameLog.Count * lineHeight );

			m_frameLogScrollPosition = GUI.BeginScrollView( frameLogRect, m_frameLogScrollPosition, frameLogScrollRect );

			for ( int i = 0; i < ms_frameLog.Count; i++ ) {
				GUI.Label( new Rect( 0, i * lineHeight, frameLogScrollRect.width, lineHeight ), ms_frameLog[ i ] );
			}

			GUI.EndScrollView( );

			if ( m_commandHistory.Count != 0 ) {

				if ( Event.current.isKey && Event.current.type == EventType.keyDown && Event.current.keyCode == KeyCode.UpArrow ) {

					m_command = m_commandHistory[ m_commandHistoryIndex ];

					m_commandHistoryIndex--;

					if ( m_commandHistoryIndex < 0 ) {
						m_commandHistoryIndex = 0;
					}

				} else if ( Event.current.isKey && Event.current.type == EventType.keyDown && Event.current.keyCode == KeyCode.DownArrow ) {

					m_commandHistoryIndex++;

					if ( m_commandHistoryIndex > m_commandHistory.Count - 1 ) {
						m_commandHistoryIndex = m_commandHistory.Count - 1;
					}

					m_command = m_commandHistory[ m_commandHistoryIndex ];
				}
			}

			if ( Event.current.isKey && Event.current.type == EventType.keyDown && Event.current.keyCode == KeyCode.Escape ) {
				ms_opened = !ms_opened;
			}

			GUI.SetNextControlName( "commandField" );
			m_command = GUI.TextArea( new Rect( 0, Screen.height * 0.5f - lineHeight, Screen.width, lineHeight ), m_command, GUI.skin.textField );
			GUI.FocusControl( "commandField" );
		}
	}

	public static bool listCommands( string s ) {

		foreach ( string key in ms_commands.Keys ) {
			DebugConsole.log( key );
		}

		return false;
	}

	public static void execute( string command, string parameters = null ) {

		if ( ms_commands.ContainsKey( command ) ) {
			ms_commands[ command ]( parameters );
		}
	}

	public static void showConsole( ) {
		ms_opened = true;
	}

	public static void hideConsole( ) {
		ms_opened = false;
	}
}
